@@ -347,6 +347,9 @@ class Backend(dbus.service.Object):
                 else:
                     safe_tar_extract(fishie, assembly_tmp)
                     logging.debug("_process_driver_fish: Extracting tar fishie %s", fishie)
+                    pre_package = os.path.join(assembly_tmp, 'prepackage.dell')
+                    if os.path.exists(pre_package):
+                        os.remove(pre_package)
             else:
                 logging.debug("_process_driver_fish: ignoring fishie %s", fishie)
 
@@ -993,6 +996,46 @@ class Backend(dbus.service.Object):
                 #Copy boot section for ISO to somewhere writable
                 shutil.copytree(os.path.join(mntdir, 'isolinux'), os.path.join(tmpdir, 'isolinux'))
 
+        #if we have any any ISO/USB bootable bootloader on the image, copy in a theme
+        grub_theme = False
+        for topdir in [mntdir, tmpdir]:
+            for bottomdir in ['i386-pc', 'x86_64-efi']:
+                if os.path.exists(os.path.join(topdir, 'boot', 'grub', bottomdir)):
+                    grub_theme = True
+        if grub_theme:
+            if not os.path.exists(os.path.join(tmpdir, 'boot', 'grub')):
+                os.makedirs(os.path.join(tmpdir, 'boot', 'grub'))
+            #conffiles
+            shutil.copy('/usr/share/dell/grub/theme/grub.cfg',
+                        os.path.join(tmpdir, 'boot', 'grub', 'grub.cfg'))
+            genisoargs.append('-m')
+            genisoargs.append(os.path.join(mntdir,'boot/grub/grub.cfg'))
+            for bottomdir in ['i386-pc', 'x86_64-efi']:
+                directory = os.path.join(mntdir, 'boot', 'grub', bottomdir)
+                if os.path.exists(directory):
+                    if not os.path.exists(os.path.join(tmpdir, 'boot', 'grub', bottomdir)):
+                        os.makedirs(os.path.join(tmpdir, 'boot', 'grub', bottomdir))
+                    shutil.copy('/usr/share/dell/grub/theme/%s/grub.cfg' % bottomdir,
+                                os.path.join(tmpdir, 'boot', 'grub', bottomdir, 'grub.cfg'))
+                    genisoargs.append('-m')
+                    genisoargs.append(os.path.join(mntdir,'boot/grub/%s/grub.cfg' % bottomdir))
+            #theme
+            if not os.path.exists(os.path.join(mntdir, 'boot', 'grub', 'dell')):
+                shutil.copytree('/usr/share/dell/grub/theme/dell', 
+                                os.path.join(tmpdir, 'boot', 'grub', 'dell'))
+            #fonts
+            if not os.path.exists(os.path.join(mntdir, 'boot', 'grub', 'dejavu-sans-12.pf2')):
+                ret = subprocess.call(['grub-mkfont', '/usr/share/fonts/truetype/ttf-dejavu/DejaVuSans.ttf',
+                                       '-s=12', '--output=%s' % os.path.join(tmpdir, 'boot', 'grub', 'dejavu-sans-12.pf2')])
+                if ret is not 0:
+                    raise CreateFailed("Creating GRUB fonts failed.")
+
+            if not os.path.exists(os.path.join(mntdir, 'boot', 'grub', 'dejavu-sans-bold-14.pf2')):
+                ret = subprocess.call(['grub-mkfont', '/usr/share/fonts/truetype/ttf-dejavu/DejaVuSans-Bold.ttf', 
+                                       '-s=14', '--output=%s' % os.path.join(tmpdir, 'boot', 'grub', 'dejavu-sans-bold-14.pf2')])
+                if ret is not 0:
+                    raise CreateFailed("Creating GRUB fonts failed.")
+
         #if we previously backed up a grub.cfg or common.cfg
         for path in ['factory/grub.cfg', 'factory/common.cfg']:
             if os.path.exists(os.path.join(mntdir, path + '.old')):
