@@ -26,6 +26,7 @@
 from ubiquity.plugin import InstallPlugin, Plugin, PluginUI
 from ubiquity import misc
 from threading import Thread
+import time
 from Dell.recovery_threading import ProgressBySize
 import debconf
 import Dell.recovery_common as magic
@@ -155,13 +156,10 @@ class PageGtk(PluginUI):
             self.hdd_recovery = builder.get_object('hdd_recovery')
             self.hdd_recovery_box = builder.get_object('hdd_recovery_box')
             self.hidden_radio = builder.get_object('hidden_radio')
-            self.reboot_dialog = builder.get_object('reboot_dialog')
-            self.reboot_dialog.set_title('Dell Recovery')
-            self.dual_dialog = builder.get_object('dual_dialog')
-            self.dual_dialog.set_title('Dell Recovery')
             self.info_box = builder.get_object('info_box')
             self.info_spinner = Gtk.Spinner()
             builder.get_object('info_spinner_box').add(self.info_spinner)
+            self.restart_box = builder.get_object('restart_box')
             self.err_dialog = builder.get_object('err_dialog')
 
             #advanced page widgets
@@ -277,13 +275,6 @@ class PageGtk(PluginUI):
                 self.err_dialog.hide()
                 return
 
-            self.controller.toggle_top_level()
-            if which == "reboot":
-                self.reboot_dialog.run()
-
-            elif which == DUAL_BOOT_QUESTION:
-                self.dual_dialog.run()
-
     def populate_devices(self, devices):
         """Feeds a selection of devices into the GUI
            devices should be an array of 3 column arrays
@@ -492,8 +483,7 @@ class Page(Plugin):
 
         #If we are in dynamic (dell-recovery/swap=dynamic) and small drive 
         #   or we explicitly disabled (dell-recovery/swap=false)
-        if not self.swap or (self.swap == "dynamic" and \
-                                       (self.mem >= 4 or self.disk_size <= 64)):
+        if not self.swap or (self.swap == "dynamic" and self.disk_size <= 64):
             self.log("Performing swap recipe fixup (%s, hdd: %i, mem: %f)" % \
                                         (self.swap, self.disk_size, self.mem))
             try:
@@ -670,50 +660,37 @@ class Page(Plugin):
             if umount is False:
                 raise RuntimeError, ("Error unmounting utility partition post-explosion.")
 
-
-    def boot_rp(self):
-        """reboots the system"""
-        with open ('/proc/cmdline', 'r') as rfd:
-            noprompt = 'noprompt' in rfd.readline()
-
-        #only cache casper if it's not going to ask the user to eject the media
-        if noprompt:
-            #Set up a listen for udisks to let us know a usb device has left
-            subprocess.call(['/etc/init.d/casper', 'stop'])
-
-            bus = dbus.SystemBus()
-            bus.add_signal_receiver(reboot_machine, 'DeviceRemoved', 'org.freedesktop.UDisks')
-
-        if self.dual:
-            dialog = DUAL_BOOT_QUESTION
-        else:
-            dialog = "reboot"
-
-        self.ui.show_dialog(dialog)
-        
-        reboot_machine(None)
-
-    def unset_drive_preseeds(self):
-        """Unsets any preseeds that are related to setting a drive"""
-        for key in [ 'partman-auto/init_automatically_partition',
-                     'partman-auto/disk',
-                     'partman-auto/expert_recipe',
-                     'partman-basicfilesystems/no_swap',
-                     'grub-installer/only_debian',
-                     'grub-installer/with_other_os',
-                     'grub-installer/bootdev',
-                     'grub-installer/make_active',
-                     'oem-config/early_command',
-                     'oem-config/late_command',
-                     'dell-recovery/active_partition',
-                     'dell-recovery/fail_partition',
-                     'ubiquity/poweroff',
-                     'ubiquity/reboot' ]:
+    def usb_boot_preseeds(self, more_keys=None):
+        """Sets/unsets preseeds that are common to a USB boot scenario.
+           This can either happen if booted from USB stick while in stage 2
+           or if booted from USB stick in stage 1 and choosing to only restore
+           the linux partition
+        """
+        keys = ['ubiquity/poweroff', 'ubiquity/reboot']
+        if more_keys:
+            keys += more_keys
+        for key in keys:
             self.db.fset(key, 'seen', 'false')
             self.db.set(key, '')
         self.db.set('ubiquity/partman-skip-unmount', 'false')
         self.db.set('partman/filter_mounted', 'true')
 
+    def unset_drive_preseeds(self):
+        """Unsets any preseeds that are related to setting a drive"""
+        keys = [ 'partman-auto/init_automatically_partition',
+                 'partman-auto/disk',
+                 'partman-auto/expert_recipe',
+                 'partman-basicfilesystems/no_swap',
+                 'grub-installer/only_debian',
+                 'grub-installer/with_other_os',
+                 'grub-installer/bootdev',
+                 'grub-installer/make_active',
+                 'oem-config/early_command',
+                 'oem-config/late_command',
+                 'dell-recovery/active_partition',
+                 'dell-recovery/fail_partition']
+        self.usb_boot_preseeds(keys)
+
     def fixup_recovery_devices(self):
         """Discovers the first hard disk to install to"""
         bus = dbus.SystemBus()
@@ -769,6 +746,7 @@ class Page(Plugin):
             location = ISO_MOUNT
         else:
             location = CDROM_MOUNT
+
         early = '/usr/share/dell/scripts/oem_config.sh early %s' % location
         self.db.set('oem-config/early_command', early)
         self.db.set('partman-auto/disk', self.device)
@@ -822,10 +800,12 @@ class Page(Plugin):
         #If we were preseeded to dynamic, look for an RP
         rec_part = magic.find_factory_rp_stats()
         if rec_type == 'dynamic':
-            if rec_part and rec_part["slave"] in mount:
+            # we rebooted with no USB stick or DVD in drive and have the RP
+            # mounted at /cdrom
+            if rec_part.has_key("slave") and rec_part["slave"] in mount:
                 self.log("Detected RP at %s, setting to factory boot" % mount)
                 rec_type = 'factory'
-            if not rec_part:
+            else:
                 self.log("No (matching) RP found.  Assuming media based boot")
                 rec_type = 'dvd'
 
@@ -1128,7 +1108,7 @@ class Page(Plugin):
                 self.rp_builder.join()
                 if self.rp_builder.exception:
                     self.handle_exception(self.rp_builder.exception)
-                self.boot_rp()
+                reboot_machine(None)
 
             # User recovery - resizing drives
             elif rec_type == "interactive":
@@ -1192,7 +1172,7 @@ class RPbuilder(Thread):
            but those would require extra dependencies, and are generally more complex
            than necessary for what needs to be accomplished here."""
 
-        white_pattern = re.compile('.')
+        black_pattern = re.compile('casper-rw|casper-uuid')
 
         #Things we know ahead of time will cause us to error out
         if self.disk_layout == 'gpt':
@@ -1226,7 +1206,7 @@ manually to proceed.")
         grub_part = STANDARD_RP_PARTITION
 
         #Calculate RP size
-        rp_size = magic.white_tree("size", white_pattern, CDROM_MOUNT)
+        rp_size = magic.black_tree("size", black_pattern, CDROM_MOUNT)
         #in mbytes
         rp_size_mb = (rp_size / 1000000) + cushion
 
@@ -1383,7 +1363,7 @@ manually to proceed.")
 
         #Copy RP Files
         with misc.raised_privileges():
-            magic.white_tree("copy", white_pattern, CDROM_MOUNT, '/mnt')
+            magic.black_tree("copy", black_pattern, CDROM_MOUNT, '/mnt')
 
         self.file_size_thread.join()
 
@@ -1441,6 +1421,8 @@ manually to proceed.")
                            '/mnt/factory/boot.img']
         for item in grub_files:
             if not os.path.exists(item):
+                os.environ['TARGET_GRUBCFG'] = '/dev/null'
+                os.environ['ISO_LOADER'] = '/dev/null'
                 build = misc.execute_root('/usr/share/dell/grub/build-binaries.sh')
                 if build is False:
                     raise RuntimeError, ("Error building grub binaries.")
@@ -1448,6 +1430,10 @@ manually to proceed.")
                     magic.white_tree("copy", re.compile('.'), '/var/lib/dell-recovery', '/mnt/factory')
                 break
 
+        #set install_in_progress flag
+        with misc.raised_privileges():
+            magic.fetch_output(['grub-editenv', '/mnt/factory/grubenv', 'set', 'install_in_progress=1'])
+
         #Install grub
         self.status("Installing GRUB", 88)
         if self.efi:
@@ -1501,23 +1487,6 @@ manually to proceed.")
                 shutil.copy(os.path.join('/tmp', files[item]), \
                             os.path.join('/mnt', 'factory', files[item]))
 
-
-        #Build new UUID
-        if int(self.mem) >= 1: #GB
-            if os.path.isdir(ISO_MOUNT):
-                syslog.syslog("Skipping UUID generation - booted from ISO image.")
-            else:
-                self.status("Regenerating UUID / initramfs", 90)
-                with misc.raised_privileges():
-                    magic.create_new_uuid(os.path.join(CDROM_MOUNT, 'casper'),
-                            os.path.join(CDROM_MOUNT, '.disk'),
-                            os.path.join('/mnt', 'casper'),
-                            os.path.join('/mnt', '.disk'))
-        else:
-            #The new UUID just fixes the installed-twice-on-same-system scenario
-            #most users won't need that anyway so it's just nice to have
-            syslog.syslog("Skipping casper UUID build due to low memory")
-
         #update bto.xml
         path = os.path.join(CDROM_MOUNT, 'bto.xml')
         if os.path.exists(path):
@@ -1545,6 +1514,11 @@ manually to proceed.")
             self.xml_obj.write_xml('/mnt/bto.xml')
         misc.execute_root('umount', '/mnt')
 
+        for count in range(100,0,-10):
+            self.status("Restarting in %d seconds." % int(count/10), count)
+            time.sleep(1)
+
+
     def exit(self):
         """Function to request the builder thread to close"""
         pass
